/*
 * Ksaf security framework label management component
 *
 * Authors:  wangshengye <wangshengye@kylinos.cn>
 *
 * Copyright (c) KylinSoft Co., Ltd. [2024].All rights reserved.
 */

#ifndef __KSAF_LABELMGR_H
#define __KSAF_LABELMGR_H

#include <string>
#include <map>
#include <vector>
#include "ksaf_common.h"

#ifdef __cplusplus
extern "C" {
#endif

	/**
	 * @description: 获取软件包的sid/oid
	 * @param {char*} pkgname 软件包名称
	 * @param {uint64_t} *sid 获取到的sid/oid
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int getPkgId(const char* pkgname, KSAF_ID* sid);
	int getPkgIds(const char* const* pkgnames, const uint32_t count, uint64_t* pkgids);

	int setPkgId(const char* pkgname, const uint64_t pkgid);
	int queryUserSetPkgId(const char* pkgname, uint64_t* pkgid);
	int showUserSetPkgId(KSAF_ID_PAIR** idPair);
	int delUserSetPkgId(const char* pkgname);
	int clearUserSetPkgId();

	int setLabelForPkg(const char** files, int size, const char* pkgname);

	/**
	 * @description: 获取文件oid
	 * @param {char*} filepath 文件路径
	 * @param {uint64_t} *oid 获取到的oid
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int getFileOid(const char* filepath, KSAF_ID* oid);
	//int getFileOids(const char* const* filepaths, const uint32_t count, KSAF_ID* oids);

	/**
	 * @description: 为文件指定oid
	 * @param {char*} filepath 文件路径
	 * @param {uint64_t} oid 要设置的oid
	 * @param {int32_t} pattern_match 是否对文件路径进行模式匹配
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int setFileOid(const char* filepath, const KSAF_ID oid, const int32_t pattern_match);

	int queryUserSetFileOid(const char* filepath, KSAF_ID_PAIR** oidPair, const int32_t pattern_match);
	int showUserSetFileOid(KSAF_ID_PAIR** oidPair);


	/**
	 * @description: 删除之前为文件指定的oid
	 * @param {char*} filepath 文件路径
	 * @param {int32_t} pattern_match 是否对文件路径进行模式匹配
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int delFileOid(const char* filepath, const int32_t pattern_match);

	/**
	 * @description: 删除之前指定的所有文件oid
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int clearFileOid();

	/**
	 * @description: 获取文件sid
	 * @param {char*} filepath 文件路径
	 * @param {uint64_t} *sid 获取到的sid
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int getFileSid(const char* filepath, KSAF_ID* sid);
	//int getFileSids(const char* const* filepaths, const uint32_t count, KSAF_ID* sids);

	/**
	 * @description: 为文件指定sid
	 * @param {char*} filepath 文件路径
	 * @param {uint64_t} oid 要设置的sid
	 * @param {int32_t} pattern_match 是否对文件路径进行模式匹配
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int setFileSid(const char* filepath, const KSAF_ID sid, const int32_t pattern_match);
	int queryUserSetFileSid(const char* filepath, KSAF_ID_PAIR** sidPair, const int32_t pattern_match);
	int showUserSetFileSid(KSAF_ID_PAIR** sidPair);


	/**
	 * @description:  删除之前为文件指定的sid
	 * @param {char*} filepath 文件路径
	 * @param {int32_t} pattern_match 是否对文件进行模式匹配
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int delFileSid(const char* filepath, const int32_t pattern_match);


	/**
	 * @description: 删除之前指定的所有文件sid
	 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
	 */
	int clearFileSid();

	int getFileTid(const char* filepath, uint32_t *tid);
	int setFileTid(const char* filepath, const uint32_t tid, const int32_t patternMatch);
	int queryUserSetFileTid(const char* filepath, KSAF_ID_PAIR** tidPair, const int32_t pattern_match);
	int showUserSetFileTid(KSAF_ID_PAIR** tidPair);
	int delFileTid(const char* filepath, const int32_t patternMatch);
	int clearFileTid();


	int getFileInheritFlag(const char* filepath, uint32_t *inheritFlag);
	int setFileInheritFlag(const char* filepath, const uint32_t inheritFlag, const int32_t patternMatch);
	int queryUserSetFileInheritFlag(const char* filepath, KSAF_ID_PAIR** inheritFlagPair, const int32_t pattern_match);
	int showUserSetFileInheritFlag(KSAF_ID_PAIR** inheritFlagPair);
	int delFileInheritFlag(const char* filepath, const int32_t patternMatch);
	int clearFileInheritFlag();

	/**
	 * @description: 获取标记设置状态
	 * @return {int} 0 成功 1 标记设置失败（需要重启） -1 dbus错误
	 */
	int getLabelSetStatus();
#ifdef __cplusplus
}
#endif

/**
 * @description: 获取内核模块的oid
 * @param {char*} moduleName 内核模块名称
 * @param {uint64_t} *oid 获取到的oid
 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
 */
int getKernelModuleOid(const char* moduleName, uint64_t* oid);

/**
 * @description: 批量获取内核模块的oid
 * @param {vector< std::string >} moduleNmaes 所有内核模块的名称
 * @param {std::map<std::string, uint64_t} oids 所有内核模块对应的oid
 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
 */
int getKernelModuleOids(const std::vector< std::string > moduleNmaes, std::map< std::string, KSAF_ID >& oids);

/**
 * @description: 获取设备oid
 * @param {char*} device 设备名称
 * @param {uint64_t} *oid 获取到的设备oid
 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
 */
int getDeviceOid(const char* device, uint64_t* oid);

/**
 * @description: 批量获取设备的oid
 * @param {vector< std::string >} devices 所有设备的名称
 * @param {std::map<std::string, uint64_t} oids 所有设备对应的oid
 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
 */
int getDeviceOids(const std::vector< std::string > devices, std::map< std::string, KSAF_ID >& oids);

/**
 * @description: 系统全盘初始化打标
 * @param {char*} filepath 文件目录路径
 * @return {void}
 */
void addTagFullSystem(const char* filepath, std::vector<std::string> excludedDir);

/**
 * @description: 计算文件/目录路径的sm3哈希值
 * @param {char*} filepath 文件/目录路径
 * @return {std::string} 哈希值字符串
 */
std::string getPathSm3HashValue(const char* filepath);

/**
 * @description: 批量获取文件/目录/符号链接的oid/sid
 * @param {std::map<std::string, std::vector<std::string>>} origin_policy 输入参数，传入主体和客体对应的文件路径
 * @return {std::map<std::string, std::map<std::string, std::vector<uint64_t>>>} extended_policy
 * 			输出参数，输出传入的主体和客体对应的文件的sid和oid
 * @return {int} 0 成功 -1 参数错误 -2 dbus错误
 */
int policies_parse(const std::map<std::string, std::vector<std::string>>& origin_policy,
	std::map<std::string, std::map<std::string, std::vector<KSAF_ID>>>& extended_policy);

/**
 * @description: 文件标记恢复（恢复到下完策略的状态）
 * @param {string} &filepath 文件路径
 * @return {std::string} 恢复后的文件标记
 */
std::string restoreSingleFileLabel(const std::string& filepath);

/**
 * @description: 文件标记恢复到初始化状态
 * @param {string} &filepath  文件路径
 * @return {std::string} 恢复后的文件标记
 */
std::string restoreSingleFileInitLabel(const std::string& filepath);

/**
 * @description: 创建一个消息订阅者
 * @return {void*} 返回消息订阅句柄指针
 */
void* createSubscriber();

/**
 * @description: 订阅标记变更消息
 * @param {void*} sub 消息订阅句柄
 * @param {string} &topic 订阅主题
 * @return {int} 返回消息订阅是否成功，0成功，-1失败
 */
int subscribeLabelChangeMsg(void* sub, const std::string& topic);

/**
 * @description: 接收订阅的标记变更消息
 * @param {void*} sub 消息订阅句柄
 * @param {char**} msg 接收消息的字符串指针
 * @return {int} 返回消息订阅是否成功，0成功，-1失败
 */
int recvLabelChangeMsg(void* sub, char** msg);

#endif //__KSAF_LABELMGR_H
